/******************************************************************************
*
* project name: DISSOLVE
* initial date: 26/06/2000
* authors:      thomas.nussbaumer@gmx.net
*               xdanger1@caramail.com
* description:  picture dissolve effect demonstration
*
* !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
* NOTE: depends on TIGCC 0.93 and later to compile
* !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
*
* for TI89 compile with:   tigcc -O3 -DUSE_TI89  dissolve.c
* for TI92P compile with:  tigcc -O3 -DUSE_TI92P dissolve.c
*
* -----------------------------------------------------------------------------
* Revision History
* -----------------------------------------------------------------------------
*
* v1.20: 18/02/2002
*        minor adaptions to TIGCC 0.93            [xdanger]
*        picture replaced                         [tnussb]
*        size reduced using static function trick [tnussb]
*
* v1.00: 26/06/2000
*        initial version
*
*******************************************************************************/

//------------------------------------------------------------------------------
// based on code from Mike Morton (Honolulu,Hawaii)
// published in: Graphics Gems (Acadamic Press 1990)
//------------------------------------------------------------------------------

#if defined(USE_TI89) && defined(USE_TI92P)
#error cannot produce both versions at once
#elif defined(USE_TI89)
#define C89_92(x,y) (x)
#elif defined(USE_TI92P)
#define C89_92(x,y) (y)
#else
#error Please specify either -DUSE_TI89 or -DUSE_TI92P on commandline
#endif


#define SAVE_SCREEN         // restore screen after program end
#define OPTIMIZE_ROM_CALLS  // optimize size
#define NO_EXIT_SUPPORT     // we don't need the exit support -> smaller code

#include <tigcclib.h>

#include "gfxdata.h" // data of picture including decompression routine


/*===========================================================================*/
/* dissolving routine                                                        */
/*                                                                           */
/* this effect uses a sequence generator which generates all possible        */
/* 15-bit numbers in a randomized way.                                       */
/*                                                                           */
/* The lower 3 bits are used as bitmask and the upper 12 bits are used as    */
/* byte offset of a pixel within the LCD memory.                             */
/*                                                                           */
/* The sequence generator is initialized with number 1. If number 1 is       */
/* generated again by the sequence generator all 15bit numbers have been     */
/* generated once !!                                                         */
/*                                                                           */
/* Normally dissolving is used to clear a screen or to fade-in a picture.    */
/* For demonstration purposes my implementation just inverts the picture.    */
/*                                                                           */
/* The XOR value (0x6000) generates all 15bit numbers. Here is a list of     */
/* XOR values for other sequences:                                           */
/*                                                                           */
/* BITWIDTH  MASK                                                            */
/*                                                                           */
/*    2      0x03                                                            */
/*    3      0x06                                                            */
/*    4      0x0C                                                            */
/*    5      0x14                                                            */
/*    6      0x30                                                            */
/*    7      0x60                                                            */
/*    8      0xB8                                                            */
/*    9      0x0110                                                          */
/*   10      0x0240                                                          */
/*   11      0x0500                                                          */
/*   12      0x0CA0                                                          */
/*   13      0x1B00                                                          */
/*   14      0x3500                                                          */
/*   15      0x6000      <--- used in this demo                              */
/*   16      0xB400                                                          */
/*   17      0x012000                                                        */
/*   18      0x020400                                                        */
/*   19      0x072000                                                        */
/*   20      0x090000                                                        */
/*   21      0x140000                                                        */
/*   22      0x300000                                                        */
/*   23      0x400000                                                        */
/*   24      0xD80000                                                        */
/*                                                                           */
/* In general the code for a "randomized" sequence generator will looks like */
/* this (SOMETYPE is an unsigned variable type which is long enough to hold  */
/* the generated values):                                                    */
/*                                                                           */
/* SOMETYPE seq = 1;                                                         */
/* do {                                                                      */
/*     if (seq & 1) seq = (seq>>1) ^ XOR_MASK;                               */
/*     else         seq = seq>>1;                                            */
/* }                                                                         */
/* while (seq != 1);                                                         */
/*                                                                           */
/*                                                                           */
/* !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! */
/*                                                                           */
/* NOTE: an important optimization hint is to define internal routines which */
/*       are used just once or twice static.                                 */
/*       This enables the optimizer to completely inline them !!!!           */
/*                                                                           */
/*===========================================================================*/
static void dissolve(void) {
	unsigned short  seq = 1;
	unsigned char   pixmask;
	unsigned short  byteoffset;

	do {
		pixmask    = seq & 0x7;
		byteoffset = seq >> 3;

        if (byteoffset < LCD_SIZE) {
            pixmask = 0x80 >> pixmask;

            //-------------------------------------
            // if pixel in plane1 is set, delete it
            // otherwise set it
            //-------------------------------------
        	if (*(unsigned char *)(GetPlane(LIGHT_PLANE)+byteoffset) & pixmask) {
          	    *(unsigned char *)(GetPlane(LIGHT_PLANE)+byteoffset) &= ~pixmask;
            }
        	else {
          	    *(unsigned char *)(GetPlane(LIGHT_PLANE)+byteoffset) |= pixmask;
            }

            //-------------------------------------
            // if pixel in plane2 is set, delete it
            // otherwise set it
            //-------------------------------------
        	if (*(unsigned char *)(GetPlane(DARK_PLANE)+byteoffset) & pixmask) {
          	    *(unsigned char *)(GetPlane(DARK_PLANE)+byteoffset) &= ~pixmask;
          	}
        	else {
          	    *(unsigned char *)(GetPlane(DARK_PLANE)+byteoffset) |= pixmask;
            }

        }

        if (seq & 1) seq = (seq>>1)^0x6000;
        else         seq = seq>>1;
    }
    while (seq != 1);
}


/*===========================================================================*/
/* main routine: where all the fun starts ...                                */
/*===========================================================================*/
void _main(void) {
    short j;

    //-------------------------------------
    // turn on grayscale graphics subsystem
    //-------------------------------------
    if (!GrayOn()) return;

     //----------------------------------------------------------------
    // clear bitplanes (fill with black) and decompress picture to them
    //-----------------------------------------------------------------
    memset(GetPlane(LIGHT_PLANE),0xff,LCD_SIZE);
    memset(GetPlane(DARK_PLANE), 0xff,LCD_SIZE);

    Decompress(((unsigned char*)GetPlane(LIGHT_PLANE))+C89_92(0,425),data1,160*100/8,20,10);
    Decompress(((unsigned char*)GetPlane(DARK_PLANE)) +C89_92(0,425),data2,160*100/8,20,10);

    //-----------------------------------------------------------
    // wait a little bit and do the dissolve/XOR effect treatment
    //-----------------------------------------------------------
    do {
        for (j=0;j<5000;j++) if (kbhit()) break;
        dissolve();
    }
    while (!kbhit());
    ngetchx();

    //---------------------------------------------
    // cleanup and bye =8-)
    //---------------------------------------------
    GrayOff();
}
